#include <stdio.h>
#include <stdlib.h>
#include "structures.h"
#include "competence.h"

/*!
* \file competence.c
* \brief Fichier contenant le code des fonctions liees aux arbres de comptences.
*/

/*! Copie de la struct Competence pour travailler

typedef struct Competence{
    char nomComp[TAILLE_NOMS];
    char id[TAILLE_ID];
    unsigned int prix_competence;
    struct Competence *frere;
    struct Competence *enfant;
    struct Effet_Competence *effet;
    boolean connu;
}Competence;

*/

/* Arbre manuel */
Competence comp_1;
Competence comp_11;
Competence comp_12;
Competence comp_111;
Competence comp_112;
Competence comp_121;

/*!
 * \fn int set_Enfant(Competence *parent, Competence *enfant)
 * \brief La fonction ajoute un enfant  une competence.
 *
 * \return Un entier.
 */
int set_Enfant(Competence *parent, Competence *nouvel_enfant)
{
    if(estVide(parent))
    {
        parent->enfant = nouvel_enfant;
    }else{
        Competence *enfant = parent->enfant;
        while(enfant!=NULL)
        {
            enfant = enfant->frere;       // A FINIR
        }
        enfant->frere = nouvel_enfant;

    }
    return 0;
}



/*!
 * \fn int init_Competence(Competence *noeud)
 * \brief La fonction initialise une competence.
 *
 * \return Un entier.
 */
int init_Competence(Competence *noeud)
{
    noeud->nomComp[0] = '\0';
    noeud->id[0] = '\0';
    noeud->prix_competence = 0;
    noeud->frere = NULL;
    noeud->enfant = NULL;
    noeud->effet = NULL;

    return 0;
}



/*!
 * \fn int taille(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants totaux d'une competence.
 *
 * \return Un entier positif.
 */
int taille(Competence *noeud)
{
    Competence *enfant;
    int compteur = 1;
    for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        compteur += taille(enfant);
    return compteur;
}


/*!
 * \fn unsigned int hauteur(Competence *noeud)
 * \brief La fonction indique la hauteur d'une competence dans l'arbre.
 *
 * \return Un entier positif.
 */
unsigned int hauteur(Competence *noeud)
{
    if (!estVide(noeud))
    {
        int compteur = 1;
        Competence *enfant;
        for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        {
            if (hauteur(enfant) > compteur)
            compteur = 1 + hauteur(enfant);
        }
        return compteur;
    }else{
        return 0;
    }
}

/*!
 * \fn int estVide(Competence *c)
 * \brief La fonction indique si une competence n'a pas d'enfants.
 *
 * \return Un booleen.
 */
int estVide(Competence *c)
{
    if (c->enfant == NULL)
        return 0;
    else
        return 1;
}


/*!
 * \fn boolean est_connue(Competence *c)
 * \brief La fonction indique si une competence est connue ou non.
 *
 * \return Un booleen.
 */
boolean est_connue(Competence *c)
{
    return (c->connu);
}


/*!
 * \fn unsigned int * NbEnfants(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants d'une competence.
 *
 * \return Un entier positif.
 */
unsigned int NbEnfants(Competence *noeud)
{
    unsigned int nb;
	if (estVide(noeud))
    {
        nb = 0;
    }
    else
    {
        Competence *enfant;
        for(noeud->enfant; enfant!= NULL; enfant=enfant->frere);

		nb++;
    }
    return nb;
}


/* -------------------------------------------------------------------------------------- */

Competence* construire_arbre_competence(Competence* a){
    return NULL;

}

int rafraichir_competences(Competence * a){
    return 0;
}



/*!
 * \fn boolean * est_complet(Competence *noeud)
 * \brief La fonction verifie si l'arbre est complet ou non.
 *
 * \return Un booleen.
 */
boolean est_complet(Competence *noeud)
{
    Competence *enfant;
    for(noeud->enfant; enfant!= NULL; enfant=enfant->frere)
    {
        if(!est_complet(enfant))
            return 0;
    }
    return 1;
}

int chemin_auto_competence(Competence* c){      /* Renvoie la liste des comptences non connues se trouvant sur le chemin d'une comptence slectionne */
    return 0;
}

Effet_Competence * get_effet(Competence *c){      /* Renvoie l'effet d'une competence */
    return c->effet;
}
